# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2021/1/21 18:15
# __fileName__ : GoldenCoordinateV2 setup.py.py
# __devIDE__ : PyCharm


"""


    Packages and dependencies

    除了建立基础的元数据信息，setup方法还需要项目中需要包含的代码块信息或
    执行该包时需要的环境（依赖的第三方包）

    Apart from establishing the basic metadata, setup() needs
information about the actual code that needs to be included, or
the environment that needs to be present on the system for this
package to be executed.

    使用packages参数可以定义我们项目中需要包含进来的模块，这里有qtictactoe和qtictactoe.images
    例如: packages=['qtictactoe', 'qtictactoe.images']
    The first keyword we need to deal with here is packages, which
defines the modules in our project that need to be included:
packages=['qtictactoe', 'qtictactoe.images'],

    注意，我们需要同时显式包含qtictactoe模块和qtictactoe.images模块
    尽管qtictactoe.images模块在qtictactoe目录下，但是它不会自动被包含在内
    Note that we need to include both the qtictactoe module and the
qtictactoe.images module explicitly; even though the images
directory is under qtictactoe, it will not be included
automatically.

    如果我们有很多子模块，但是不想手动显式包含在内，setuptools工具提供了一种自动解决方案
    即使用find_packages()方法来在该项目根路径下搜索所有的模块
    想要使用find_packages()方法来查找所有的模块，则必须确保子模块有一个__init__.py文件
    这样setupUtils才能将该目录识别为模块
    If we had a lot of submodules and didn’t want to explicitly list
them, setuptools offers an automatic solution as well:
            from setuptools import setup, find_packages
                setup(
                #...
                packages=find_packages(),
                )
    If you want to use find_packages, make sure each submodule has an
__init__.py file in it so that setuputils can identify it as a module. In
this case, you’d need to add an __init__.py file to the images folder or it
will be ignored.

    手动导入模块需要更多的工作，而find_packages方法可能会在某些情况下对类库模块识别失败
    Both approaches have advantages and drawbacks; the manual
approach is more work, but find_packages may sometimes fail to
identify a library in certain circumstances.

    我们也需要指定在项目中使用到的第三方依赖库，则配置install_requires参数
    We also need to specify the external libraries that are needed
for this project to run—in this case, PySide2. That can be done
with the install_requires keyword

    这个参数是列表形式，指定该项目中使用到的全部第三方类库，该案例中则是PySide2，
    就像requirements.txt文件那样，可以显式指定依赖库的版本号:
    install_requires=['PySide2 >= 5.12'],不指定则默认为最新版本
    This keyword takes a list of names for packages that must be
installed for the program to be installed. When your program
is installed using pip, it will use this list to install all
dependency packages automatically. You should include
anything that isn’t part of the standard library in this list.
    Just like the requirements.txt file, we can even be explicit about
the version number of each dependency required:
install_requires=['PySide2 >= 5.12'],
    In this case, pip would make sure a version of PySide2 greater
than or equal to 5.12 is installed. If no version is specified, pip
will install the latest version available from PyPI.


    在某些情况下，我们可能还需要指定Python的版本，比如我们的项目使用了f-strings的语法，
    这个功能只在Python3.6+以上版本才能使用，所以我们需要指定Python的版本，配置python_requires参数
    In some cases, we might require a certain version of Python as
well; for example, our project uses f-strings, a feature only
found in Python 3.6 or greater. We can specify that with the
python_requires keyword:

    我们还可以为可选功能指定依赖，比如，如果我们给qtictactoe添加一个可选的network-play功能
    且该功能依赖requests类库，则可以如下指定extras_require={"NetworkPlay": ["requests"]}
    这个extras_require参数接收一个map字典，key是功能名，value则是该功能所依赖的一系列包名
    We can also specify dependencies for optional features; for
example, if we added an optional network-play feature to
qtictactoe, which required the requests library, we would specify
that like so:
            extras_require={
                "NetworkPlay": ["requests"]
            }
    The extras_require keyword accepts a mapping of feature names
(which can be anything you want) to lists of package names.


    这些额外功能所对应的依赖包不会在主模块被安装时自动安装，但是当其他模块引用这些子功能时，
    则会触发相应的依赖被安装，比如另一个模块可以指定我们项目下的NetworkPlay额外功能，
    install_requires=['QTicTacToe[NetworkPlay]']，此时就会安装requests类库
    These modules won’t be automatically installed when your
package is, but other modules can depend on these
subfeatures. For example, another module can specify a
dependency on our project’s NetworkPlay extra keywords like so:
install_requires=['QTicTacToe[NetworkPlay]'],
    This would trigger a cascade of dependencies that would result
in the requests library being installed.


        Non-Python files

        默认情况下，setuptools将会打包我们项目下它能发现的所有Python文件，并且其他类型
        的文件将会被忽略，但是在大多数项目中，我们希望非Python文件也能被包含在内，一起打包
        这些非Python文件大致可以分为两类
            1. Python模块的组成部分，比如PNG文件
            2. 还有一些非模块相关文件，比如README文件

        By default, setuptools will package the Python files it finds in
    our project, and other file types will be ignored. In almost any
    project, however, there are going to be non-Python files that
    we’re going to want to include in our distribution package.
    These files fall generally into two categories: those that are
    part of a Python module, such as our PNG files, and those that
    are not, such as the README file.

        为了合并非Python模块组成的那一些文件，我们需要创建一个名叫MANIFEST.IN的文件，这个
        文件包含有项目根路径下文件路径指令，比如我们想要把文件文件包含进来，则如下所示
        所有的文件指令都是相对于项目根路径的
        To incorporate files that are not part of a Python package, we
    need to create a file called MANIFEST.in. This file contains include
    directives for file paths underneath the project root. For
    example, if we want to include our documentation files, ours
    should look like the following:
            include README.rst
            include LICENSE
            include requirements.txt
            include docs/*
        The format is simple: the word include followed by a filename,
    path, or pattern that will match a set of files. All paths are
    relative to the project root.


    为了包含Python模块组成部分的非Python文件，我们有两个选择
        1. 包含它们的路径指令到MANIFEST.IN文件中，接着设置include_package_data=True
        2. 使用package_data关键字参数来进行配置，比如
        package_data={
            'qtictactoe.images': ['*.png'],
            '': ['*.txt', '*.rst']
        },


    To include files that are part of a Python package, we have two choices.
One way is to include them in the MANIFEST.in file, and then set include_package_data to True in setup.py:
            include_package_data=True,
    Another way to include non-Python files is to use the
            package_data keyword argument in setup.py:
            package_data={
            'qtictactoe.images': ['*.png'],
            '': ['*.txt', '*.rst']
            },

    package_data参数接收一个dict对象，每一个key都是一个模块路径，value则是要包含进来的文件的匹配模式
    在这里，我们想要包含qtictactoe.images模块下所有的PNG文件以及每一个模块下的txt、rst文件
    这个参数只能配置模块下的非Python文件
    This argument takes a dict object, where each item is a module
path and a list of patterns that match the files included. In this
case, we want to include all PNG files found in the
qtictactoe.images module, and any TXT or RST files anywhere
in the package. Keep in mind this argument only applies to
files in the module directory (that is, files under qtictactoe).
    If we want to include files such as README.rst or run.py, those
should be put in the MANIFEST.in file.

    两种方式只能使用其一，如果include_package_data=True，则package_data将会被自动忽略
    You can use either approach to including files, but you cannot use
both approaches in the same project; if you enable
include_package_data, the package_data directives will be ignored

    Executables

    我们往往将PyPi认为是一种安装Python库的工具，事实上，它也可以很好地安装应用程序，
    并且我们可以在上面获得很多Python应用，即使你正在创建一个类库，但是很大可能你的类库将附带可执行能力，
    比如PySide2的pyrcc5和pyuic5可执行工具
    We tend to think of PyPI as a tool to install Python libraries; in
fact, it works well for installing applications as well, and many
Python applications are available from it. Even if you are
creating a library, it’s likely your library will ship with
executable utilities, such as the pyrcc5 and pyuic5 utilities that
come with PySide2.

    为了具有可执行的能力，setupUtils给我们提供了一种指定特殊函数或方法作为控制台脚本的方式(使用entry_points参数进行控制)
    当package被安装时，它将会创建一个简单的可执行文件，当其被通过命令行执行时，将会调用相关函数或方法
    比如    entry_points={
                'console_scripts': [
                    'qtictactoe = qtictactoe.__main__:main'
                ]
            }




    To accommodate these, setuputils gives us a way to specify
particular functions or methods as console scripts; when the
package is installed, it will create a simple executable file,
which will call that function or method when executed from
the command line.
This is specified using the entry_points keyword:
            entry_points={
                'console_scripts': [
                    'qtictactoe = qtictactoe.__main__:main'
                ]
            }

    这个entry_points字典还有其他的用途，但是我们最关心的一个是console_scripts
    这个key指向一系列我们想要设置为控制台脚本命令的特殊函数，语法形式为: 'command_name = module.submodule:function'
    The entry_points dictionary has other uses, but the one we’re
most concerned with is the 'console_scripts' key. This key
points to a list of strings that specify the functions we want set
up as command-line scripts. The format for these strings is as
follows
        'command_name = module.submodule:function'

    我们可以添加很多控制台脚本，它们只需要指向包下的一个函数或方法，不能指向一个Python文件
    这就是为什么我们将所有执行代码封装到main函数中的原因，setuptools还包含更多指令以处理不常见的问题，
    可以参考: https://setuptools.readthedocs.io/en/latest/setuptools.html
    You can add as many console scripts as you wish; they just
need to point to a function or method in the package that can
be run directly. Note that you must specify an actual callable
here; you can’t just point to a Python file to run. This is why
we’ve put all the execution code under a main() function inside
__main__.py.
    setuptools contains many more directives to deal with less
common situations; for a complete list, see https://setuptools.re
adthedocs.io/en/latest/setuptools.html
"""
from setuptools import setup, find_packages

setup(
    # Arguments here
    # Basic metadata arguments
    name='CrawlSpider',
    version='1.0',
    author='Ricky.wang',
    author_email='ricky.wbjian@gmail.com',
    description='common opration from spider',
    url="https://gitee.com/dreamricky/crawl-spider",
    license='MIT',
    keywords='spider scrapy beatifulsoup xpath 正则',
    project_urls={
        'Author Website': 'https://blog.csdn.net/qq_36154755',
    },
    packages=find_packages(),
    install_requires=['PySide2'],
    python_requires='>=3.6',
    extras_require={
        "NetworkPlay": ["requests"]
    },
    package_data={
        'CrawlSpider/File': ['*']
    },
    entry_points={
        'console_scripts': [
            'request = CrawlSpider.Utils.ConsoleScripts:main'
        ]
    }
)





